require 'graphql'

module Recline
  module Types; end

  def self.appearance_for(field, klass, name)
    if field.metadata.key? :recline_appearance
      return field.metadata[:recline_appearance]
    else
      field_type = field.type
      while field_type.kind.wraps?
        field_type = field_type.of_type
      end

      if default_appearance_types.key? field_type
        default_appearance_types[field_type]
      else
        db_type = klass.type_for_attribute(name.to_s).type
        if default_appearance_types.key? db_type
          default_appearance_types[db_type]
        else
          fallback_appearance
        end
      end
    end
  end

  class << self
    @schema_name_prefix = 'Recline'
    @fallback_appearance = :string

    attr_accessor :schema_name_prefix
    attr_accessor :fallback_appearance
    attr_accessor :default_appearance_types

    def default_appearance_types
      @default_appearance_types ||= {
        GraphQL::ID_TYPE => :string,
        GraphQL::STRING_TYPE => :string,
        GraphQL::INT_TYPE => :number,
        GraphQL::FLOAT_TYPE => :number,
        GraphQL::BOOLEAN_TYPE => :toggle,
        text: :text,
        date: :date,
        datetime: :datetime,
        timestamp: :datetime
      }
    end
  end
end

require 'recline/railtie' if defined?(Rails)
require 'recline/model'
require 'recline/attribute_reflection'
require 'recline/model_reflection'
require 'recline/appearance'
require 'recline/types/appearance_enum_type'
require 'recline/types/attribute_reflection_type'
require 'recline/types/model_reflection_type'
require 'recline/appearance'
